/***************************************************************************

    file        : raceselectmenu.cpp
    created     : Sat Nov 16 09:36:29 CET 2002
    copyright   : (C) 2002 by Eric Espie
    email       : eric.espie@torcs.org   
    version     : $Id: raceselectmenu.cpp 7827 2021-12-17 00:57:48Z iobyte $                                  

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/** @file   
    		Race selection menu
    @author	<a href=mailto:eric.espie@torcs.org>Eric Espie</a>
    @version	$Id: raceselectmenu.cpp 7827 2021-12-17 00:57:48Z iobyte $
*/

#include <map>
#include <algorithm>
#include <string>
#include <memory>

#include <tgfclient.h>
#include <robot.h>
#include <raceman.h>
#include <racemanagers.h>
#include <race.h>
#include <cars.h>
#include <drivers.h>

#include "legacymenu.h"
#include "racescreens.h"
#include "tradermenu.h"

class GfCars;
class GfDrivers;
class GfRaces;

// Menu control Ids
static int   PracticeButtonId;
static int   Challenge1ButtonId;
static int   Challenge2ButtonId;
static int   Challenge3ButtonId;
static int   Challenge4ButtonId;
static int   Challenge5ButtonId;
static int   Challenge6ButtonId;
static int   Challenge7ButtonId;

static int   Endurance1ButtonId;
static int   Endurance2ButtonId;
static int   Endurance3ButtonId;
static int   Endurance4ButtonId;

static int   Championship1ButtonId;
static int   Championship2ButtonId;
static int   Championship3ButtonId;
static int   GarageButtonId;

//bool         rmUpdate = false;
bool         rmCar          = false;
bool         rmLicence      = false;
bool         rmChallenge    = false;
bool         rmEndurance    = false;
int          rmLevel[];
double       rmFame = 0.0;

//unsigned int rmCredit = 0;

static const char* CurCarName = "";
static std::string CurChampionship;

// The Race Select menu.
void *RmCareerSelectMenuHandle = nullptr;

// The car selection menu.
static RmTraderMenu TraderMenu;
static bool running;

/* Called when the menu is activated */
static void
rmOnActivate(void * /* dummy */)
{
	GfLogTrace("Entering Race Mode Select menu\n");

    LmRaceEngine().reset();
}

/* Buy car menu */
static void
rmOnRaceSelectBuy(void *prevMenu)
{
    TraderMenu.setPreviousMenuHandle(prevMenu);
    TraderMenu.runMenu("sc-deckard-conejo");
}

/* garage car menu */
static void
rmOnRaceSelectGarage(void *prevMenu)
{
    GfuiScreenActivate(RmGarageSelectInit(RmCareerSelectMenuHandle));
}

/* Exit from Race engine */
static void
rmOnRaceSelectShutdown(void *prevMenu)
{
    GfuiScreenActivate(MenuHandle);

    LmRaceEngine().cleanup();
    LegacyMenu::self().setCareerMode(false);

    LegacyMenu::self().shutdownGraphics(/*bUnloadModule=*/true);
}

static void
rmOnSelectRaceManPractice(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCat[rmIndex]+"practice";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);
    GfLogDebug("Race Practice = %s\n", race.c_str());
    running = true;

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(false);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManChallenge1(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"challenge1";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(true);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManChallenge2(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"challenge2";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(true);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManchallenge3(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"challenge3";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(true);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManChallenge4(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"challenge4";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(true);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManChallenge5(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"challenge5";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(true);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManChallenge6(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"challenge6";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(true);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManChallenge7(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"challenge7";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(true);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManEndurance1(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"endurance1";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(false);
        LegacyMenu::self().setEnduranceMode(true);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManEndurance2(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"endurance2";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(false);
        LegacyMenu::self().setEnduranceMode(true);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManEndurance3(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"endurance3";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(false);
        LegacyMenu::self().setEnduranceMode(true);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManEndurance4(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"endurance4";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(false);
        LegacyMenu::self().setEnduranceMode(true);
        LegacyMenu::self().setChampionshipMode(false);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManChampionship1(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"championship1";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);
    GfLogDebug("#Career Championship 1 = %s\n", race.c_str());

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(false);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(true);
        std::string strBase(GfLocalDir());
        std::string strPlayersFileName = "drivers/human/human.xml";
        char sstring[256];

        void *hparmHuman = GfParmReadFileLocal(strPlayersFileName, GFPARM_RMODE_STD | GFPARM_RMODE_REREAD);
        snprintf(sstring, sizeof(sstring), "%s/%s/%d", "Robots", "index", 1);
        GfParmSetStr(hparmHuman, sstring, "championship", race.c_str());
        GfParmWriteFile(0, hparmHuman, "human");
        GfParmReleaseHandle(hparmHuman);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManChampionship2(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"championship2";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);
    GfLogDebug("#Career Championship 2 = %s\n", race.c_str());

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(false);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(true);
        std::string strBase(GfLocalDir());
        std::string strPlayersFileName = "drivers/human/human.xml";
        char sstring[256];

        void *hparmHuman = GfParmReadFileLocal(strPlayersFileName, GFPARM_RMODE_STD | GFPARM_RMODE_REREAD);
        snprintf(sstring, sizeof(sstring), "%s/%s/%d", "Robots", "index", 1);
        GfParmSetStr(hparmHuman, sstring, "championship", race.c_str());
        GfParmWriteFile(0, hparmHuman, "human");
        GfParmReleaseHandle(hparmHuman);
        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static void
rmOnSelectRaceManChampionship3(void *pvRaceManTypeIndex)
{
    // Get the race managers with the given type
    std::string race = _pCategory+"championship3";
    GfRaceManager* pSelRaceMan = GfRaceManagers::self()->getRaceManager(race);
    GfLogDebug("#Career Championship 3 = %s\n", race.c_str());

    if (pSelRaceMan)
    {
        LegacyMenu::self().setChallengeMode(false);
        LegacyMenu::self().setEnduranceMode(false);
        LegacyMenu::self().setChampionshipMode(true);
        std::string strBase(GfLocalDir());
        std::string strPlayersFileName = "drivers/human/human.xml";
        char sstring[256];

        void *hparmHuman = GfParmReadFileLocal(strPlayersFileName, GFPARM_RMODE_STD | GFPARM_RMODE_REREAD);
        snprintf(sstring, sizeof(sstring), "%s/%s/%d", "Robots", "index", 1);
        GfParmSetStr(hparmHuman, sstring, "championship", race.c_str());
        GfParmWriteFile(0, hparmHuman, "human");
        GfParmReleaseHandle(hparmHuman);

        // Give the selected race manager to the race engine.
        LmRaceEngine().selectRaceman(pSelRaceMan);

        // Start the race configuration menus sequence.
        LmRaceEngine().configureRace(/* bInteractive */ true);
    }
    else
    {
        GfLogError("No such race manager (type '%s')\n", race.c_str());
    }
}

static
void rmRaceCategoryInit()
{
    GfLogDebug("Prefix category Race : \n");
    std::string category = rmCategory;

    if (strcmp(rmCategory, "Supercars Class C") == 0)
    {
        rmIndex    = 0;
        _pCategory = _pCat[rmIndex];

    }
    else if(strcmp(rmCategory, "2009 Grand Prix 5") == 0)
    {
        rmIndex    = 1;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "Supercars Class B") == 0)
    {
        rmIndex    = 2;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "2010 Grand Prix 4") == 0)
    {
        rmIndex    = 3;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "Supercars Class A") == 0)
    {
        rmIndex    = 4;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "1991 Grand Prix") == 0)
    {
        rmIndex    = 5;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "Long Day Series GT2") == 0)
    {
        rmIndex    = 6;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "2001 Grand Prix") == 0)
    {
        rmIndex    = 7;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "Long Day Series GT1") == 0)
    {
        rmIndex    = 8;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "2001 GP American") == 0)
    {
        rmIndex    = 9;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "Long Day Series P2") == 0)
    {
        rmIndex    = 10;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "2011 GP American") == 0)
    {
        rmIndex    = 11;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "Long Day Series P1") == 0)
    {
        rmIndex    = 12;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "2012 GP American") == 0)
    {
        rmIndex    = 13;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "1936 Grand Prix") == 0)
    {
        rmIndex    = 14;
        _pCategory = _pCat[rmIndex];
    }
    else if (strcmp(rmCategory, "1967 Grand Prix") == 0)
    {
        rmIndex    = 15;
        _pCategory = _pCat[rmIndex];
    }

    GfLogDebug("Prefix category Race : %s - Ranking index = %i\n", _pCategory.c_str(), rmIndex);
}

/* Initialize the single player menu */
void*
RmCareerSelectInit(void *prevMenu)
{
    if (RmCareerSelectMenuHandle)
    {
        RmCareerSelectMenuHandle = NULL;
    }

    //std::string strBase(GfLocalDir());
    std::string strPlayersFileName = "drivers/human/human.xml";
    std::string strGarageFileName = "drivers/human/garage.xml";
    std::string strContractFilename = "drivers/human/contracts.xml";
    std::string strAiConfigFileName ="config/raceman/extra/skill.xml";

    void *hparmHuman = GfParmReadFileLocal(strPlayersFileName, GFPARM_RMODE_STD | GFPARM_RMODE_REREAD);

    if (!hparmHuman)
        hparmHuman = GfParmReadFile(strPlayersFileName, GFPARM_RMODE_STD | GFPARM_RMODE_REREAD);

    if (!hparmHuman)
    {
        GfLogError("No usable human driver (human.xml not found or not readable)\n");
    }

    if(hparmHuman)
    {
        char drvSectionPath[128];
        snprintf(drvSectionPath, sizeof(drvSectionPath), "%s/%s/%u", ROB_SECT_ROBOTS, ROB_LIST_INDEX, 1);
        const char *strSkillLevel = GfParmGetStr(hparmHuman, drvSectionPath, ROB_ATTR_LEVEL, ROB_VAL_SEMI_PRO);
        bool skill = (strcmp(strSkillLevel, "semi-pro") == 0) || (strcmp(strSkillLevel, "pro") == 0);
        int playerskill = 0.0;

        if(strcmp(strSkillLevel, "arcade") == 0)
            playerskill = 0;
        else if (strcmp(strSkillLevel, "semi-rookie") == 0)
            playerskill = 1;
        else if (strcmp(strSkillLevel, "rookie") == 0)
            playerskill = 2;
        else if (strcmp(strSkillLevel, "amateur") == 0)
            playerskill = 3;
        else if (strcmp(strSkillLevel, "semi-pro") == 0)
            playerskill = 4;
        else if (strcmp(strSkillLevel, "pro") == 0)
            playerskill = 5;

        void *paramHandle = GfParmReadFileLocal(strAiConfigFileName, GFPARM_RMODE_STD | GFPARM_RMODE_REREAD);
        float aiskill = (float)GfParmGetNum(paramHandle, "skill", "level", 0, 10.0);
        GfParmReleaseHandle(paramHandle);

        if((aiskill > 1.0) || (skill == false))
        {
            return MenuHandle;
        }

        LegacyMenu::self().setAiSkill(aiskill);
        LegacyMenu::self().setPlayerSkill(playerskill);
    }

    const char* pszCarId = "";

    rmCar = false;
    LegacyMenu::self().setCareerMode(true);
    LegacyMenu::self().setChampionshipMode(false);
    GfLogDebug("#careerselectmenu rmCareer = %i - rmChampionship = %i\n", LegacyMenu::self().getCareerMode(),
             LegacyMenu::self().getChampionshipMode());
    _pCategory = "";
    CurChampionship = "none";

    void *hparmGarage = GfParmReadFileLocal(strGarageFileName, GFPARM_RMODE_STD | GFPARM_RMODE_REREAD);

    if (!hparmGarage)
        hparmGarage = GfParmReadFile(strGarageFileName, GFPARM_RMODE_STD | GFPARM_RMODE_REREAD);

    if (!hparmGarage)
    {
        GfLogError("No usable Garage (garage.xml not found or not readable)\n");
    }

    if(hparmGarage)
    {
        char sstring[128];
        snprintf(sstring, sizeof(sstring), "%s/%s/%d", "cars", "index", 1);
        CurCarName = GfParmGetStr(hparmGarage, sstring, ROB_ATTR_GARAGE1, "none");

        if(strlen(CurCarName) == 0 || strcmp(CurCarName, "none") == 0)
            rmCar = false;
        else
        {
            rmCar = true;
        }

        GfLogDebug("Cars in Garage = %s\n", CurCarName);
    }

    if (hparmHuman)
    {
        char sstring[256];

        GfLogDebug("Loadind race select init human\n");

        snprintf(sstring, sizeof(sstring), "%s/%s/%d", "Robots", "index", 1);
        for (int i = 0; i < 18; i++)
        {
            const char *strLevel = rmlevel[i];
            rmLevel[i] = (unsigned int)GfParmGetNum(hparmHuman, sstring, strLevel, (char*)NULL, 0);
        }

        rmCredit = (unsigned long int)GfParmGetNum(hparmHuman, sstring, "credit", (char*)NULL, 10000);
        rmFame = (double)GfParmGetNum(hparmHuman, sstring, "fame", (char*)NULL, 0.0);
        rmLicence = (bool)GfParmGetNum(hparmHuman, sstring, "licence", (char*)NULL, 0);

        GfLogDebug("Career mode Level = %u - Licence = %i - Credit = %lu\n", rmLevel[rmIndex], rmLicence, rmCredit);

        if(rmCar)
        {
            pszCarId = GfParmGetStr(hparmHuman, sstring, "car career", "none");
            GfLogDebug("# Car Career = %s\n", pszCarId);

            if(!(strcmp(pszCarId, "none") == 0))
                CurChampionship = GfParmGetStr(hparmHuman, sstring, "championship", "none");
            GfLogDebug(" # championship load = %s\n", CurChampionship.c_str());
            GfParmSetStr(hparmHuman, sstring, "car name", pszCarId);

            std::string car = GfCars::self()->getCar(pszCarId)->getCategoryName();
            rmCategory = car.c_str();
            GfLogDebug("Category = %s\n", rmCategory);

            GfParmWriteFile(0, hparmHuman, "human");
            GfParmReleaseHandle(hparmHuman);

            GfParmReleaseHandle(hparmGarage);
            //GfParmReleaseHandle(hdparmcontracts);

            rmRaceCategoryInit();
        }
    }

    GfLogDebug(" # championship 2 load = %s\n", CurChampionship.c_str());
	// Ask the RaceEngine what types of races should be allowed here
    //bool SupportsHumanDrivers = LmRaceEngine().supportsHumanDrivers();

    // Create screen, load menu XML descriptor and create static controls.
    RmCareerSelectMenuHandle = GfuiScreenCreate((float*)NULL,
											NULL, rmOnActivate, 
											NULL, (tfuiCallback)NULL, 
											1);

    void *hMenuXMLDesc = GfuiMenuLoad("careermenu.xml");
    GfuiMenuCreateStaticControls(RmCareerSelectMenuHandle, hMenuXMLDesc);

    int CarImageId = GfuiMenuCreateStaticImageControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "PortraitImage");
    GfuiStaticImageSet(RmCareerSelectMenuHandle, CarImageId, "data/img/portrait8.png");

    int Text1Label = GfuiMenuCreateLabelControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Text1Label");
    int Text2Label = GfuiMenuCreateLabelControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Text2Label");
    int Text3Label = GfuiMenuCreateLabelControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Text3Label");
    int Text4Label = GfuiMenuCreateLabelControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Text4Label");
    int Race1Label = GfuiMenuCreateLabelControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Race1Label");
    int Race2Label = GfuiMenuCreateLabelControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Race2Label");

    int val = CurChampionship.compare("none");
    int index = rmLevel[rmIndex] % 10;

    GfLogDebug("# val rmLevel[%i] = %i\n", rmIndex, index);

    if (val < 0 || val > 0)
    {
        GfLogDebug(" # championship load = %s\n", CurChampionship.c_str());

        auto const pos = CurChampionship.find_last_of('-');
        const auto leaf = CurChampionship.substr(pos + 1);

        GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "");
        GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Load and Continue your championship !");
        GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "");
        GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "");

        GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Championship in run");

        if(leaf.compare("championship1") == 0)
            Championship1ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Championship1Button",
                                                                NULL, rmOnSelectRaceManChampionship1);
        else if (leaf.compare("championship2") == 0)
            Championship2ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Championship2Button",
                                                                NULL, rmOnSelectRaceManChampionship2);
        else if (leaf.compare("championship3") == 0)
            Championship3ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Championship3Button",
                                                                NULL, rmOnSelectRaceManChampionship3);

        GfLogDebug("# Championship leaf = %s\n", leaf.c_str());
    }
    else if(rmCar == false)
    {
        GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Hi pilot !");
        GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "To be able to go race");
        GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "you need a car !!!");
        GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Go buy one");
    }
    else if (index == 0)
    {
        if(running)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Sorry guy");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "you have to do better than that !!");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "go back to that track and start again");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "you will succeed !");
        }
        else if(rmIndex == 0)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Good");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Now, you have to do a test drive with your new car");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "set a lap record under 1 minute 50 seconds");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Alicante");
        }
        else if(rmIndex == 1)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Good, you enter in the OpenWheeler category");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Now, you have to do a test drive with your new car");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "set a lap record under 51 seconds");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Pinabashi");
        }
        else if(rmIndex == 2)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Welcome in Supercars Class B !!");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "you're going to try out your new car");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "set a lap record under 1 minute 43 seconds to Espie");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive on Espie");
        }
        else if(rmIndex == 3)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Ohhh !! Welcome to the last class before Grand Prix Formula");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "you bought yourself a new car ! Let's see what you can do with it");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "set a lap record under 1 minute 50 seconds to Jarama");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive on Jarama");
        }
        else if(rmIndex == 4)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Last Supercars Class before LS");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "would you be as strong as you claim ? let's see this !!");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 20 seconds to Prenzlau");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Prenzlau");
        }
        else if(rmIndex == 5)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Good, now you enter in first class grand prix formula");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "let's see if you're a great pilot");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 20 seconds to Sao-Paulo");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Sao-Paulo");
        }
        else if(rmIndex == 6)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Do you want to run in the big league ?");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "we will now be able to check what you are really worth");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 42 seconds to Forza");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Forza");
        }
        else if(rmIndex == 7)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "You bought yourself a GP One, do you think you're up to it ?");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "you will have to measure yourself against real pilots");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 30 seconds to Melbourne");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Melbourne");
        }
        else if(rmIndex == 8)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "now you are in the last category of LS-GT");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "you'll have to race against the best");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 3 minute 50 seconds to Longday");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Longday");
        }
        else if(rmIndex == 9)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Are you sure you want to race on the Speedways ?");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Be careful, these are dangerous races, accidents are not uncommon.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 30 seconds to Tokyo Speedway");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Tokyo Speedway");
        }
        else if(rmIndex == 10)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Wow !! You're in the Endurance category.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Consistency and mastery is the key");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 50 seconds to Salty");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Salty");
        }
        else if(rmIndex == 11)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Now you are in the 2011 American Grand Prix category");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "the races will be more difficult !!");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 00 seconds to Milky-Five Speedway");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Milky-Five Speedway");
        }
        else if(rmIndex == 12)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "And now the last Endurance serie.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Are you going to finish your career in style ?");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 50 seconds to Salty");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Salty");
        }
        else if(rmIndex == 13)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "the last category of the American Grand Prix");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "retirement is coming soon ...");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 00 seconds to Milky-Five Speedway");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Milky-Five Speedway");
        }
        else if(rmIndex == 14)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "you want to try your hand at the 1936 historic cars");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "good luck driving without insurance");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 50 seconds to Charmey");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Charmey");
        }
        else if(rmIndex == 15)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "The best category to drive without assistance !");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "To drive with finesse");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Set a lap record under 1 minute 00 seconds to Kurpfalzring");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Test drive at Kurpfalzring");
        }

        PracticeButtonId =
                GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "PracticeButton",
                                    NULL, rmOnSelectRaceManPractice);
    }
    else if (index == 1)
    {
        if(rmIndex == 0)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Very Good");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "There are two races available this weekend, I'll let you choose");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "which one you want to run in.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "Good luck !");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race on Alicante");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Race on Auriac");
        }
        else if (rmIndex == 1)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Good");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "There are two races available this weekend, I'll let you choose");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "which one you want to run in.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "Good luck !");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race on Alicante");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Race on Auriac");
        }

        Challenge1ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Challenge1Button",
                                                                NULL, rmOnSelectRaceManChallenge1);
        Challenge2ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Challenge2Button",
                                                                NULL, rmOnSelectRaceManChallenge2);
    }
    else if (index == 2)
    {
        if(rmIndex == 0)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "well, very well,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "There are two races available this weekend, I'll let you choose");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "First is a challenge at Peters for 10 laps");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "Second is an endurance race at Brondehach for 80 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race on Peters");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Race on Brondehach");
        }
        else if (rmIndex == 1)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Good");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "There are two races available this weekend, I'll let you choose");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "First is a challenge at Peters for 15 laps");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "Second is an endurance race at Brondehach for 80 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race on Peters");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Race on Brondehach");
        }

        Challenge3ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Challenge3Button",
                                                                NULL, rmOnSelectRaceManchallenge3);
        Endurance1ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Endurance1Button",
                                                                NULL, rmOnSelectRaceManEndurance1);
    }
    else if (index == 3)
    {
        if(rmIndex == 0)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Very good,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races may be longer in endurance categories.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Two races are available this weekend. The first on an Karwada for 10 laps,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is an endurace race at Auriac track for 160 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race on Karwada");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Race on Auriac");
        }
        else if (rmIndex == 1)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Very Good");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races may be longer in endurance categories.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Two races are available this weekend. The first on an Karwada for 10 laps,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is an endurace race at Auriac track for 160 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race on Karwada");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Race on Auriac");
        }

        Challenge4ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Challenge4Button",
                                                                NULL, rmOnSelectRaceManChallenge4);
        Endurance2ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Endurance2Button",
                                                                NULL, rmOnSelectRaceManEndurance2);
    }
    else if (index == 4)
    {
        if(rmIndex == 0)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Very good,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available this weekend.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first a challenge on Brondehach,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is a championship with six races on 80 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race on Brondehach");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Championship 80 Km");
        }
        else if (rmIndex == 1)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Very Good");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available this weekend.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first a challenge on Brondehach,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is a championship with six races on 80 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race on Brondehach");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Championship 80 Km");
        }

        Challenge5ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Challenge5Button",
                                                                NULL, rmOnSelectRaceManChallenge5);
        Championship1ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Championship1Button",
                                                                NULL, rmOnSelectRaceManChampionship1);
    }
    else if (index == 5)
    {
        if(rmIndex == 0)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "You're the best");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available this weekend.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first is a challenge on Chemisay with 10 laps,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is an endurance at Peters on 320 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race at Chemisay");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Race on peters");
        }
        else if (rmIndex == 1)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "You're the best");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available this weekend.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first is a challenge on Chemisay with 10 laps,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is an endurance at Peters on 320 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race at Chemisay");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Race on peters");
        }
        Challenge6ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Challenge6Button",
                                                                NULL, rmOnSelectRaceManChallenge6);
        Endurance3ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Endurance3Button",
                                                                NULL, rmOnSelectRaceManEndurance3);
    }
    else if (index == 6)
    {
        if(rmIndex == 0)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "You manage to surprise me");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available this weekend.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first is a challenge at Forza on 10 laps,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is championship with six races");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race on Forza Challenge");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Championship on 160 Km");
        }
        else if (rmIndex == 1)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "You manage to surprise me");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available this weekend.");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first is a challenge at Newton Speedway,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is championship with six races");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Challenge on Newton Speedway");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Championship on 160 Km");
        }

        Challenge7ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Challenge7Button",
                                                                NULL, rmOnSelectRaceManChallenge7);
        Championship2ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Championship2Button",
                                                                NULL, rmOnSelectRaceManChampionship2);
    }
    else if (index == 7)
    {
        if(rmIndex == 0)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Very good,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available :");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first on at Auriac for 10 laps,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is a endurance race at Karwada on 500 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race on Auriac");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Race on Karwada");
        }
        else if (rmIndex == 1)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Now you can do a Championship season");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available :");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first on at Auriac for 10 laps,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is a endurance race on Karwada on 500 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Challenge at Auriac");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Endurance at Karwada on 500 Km");
        }

        Challenge2ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Challenge2Button",
                                                                NULL, rmOnSelectRaceManChallenge2);
        Endurance4ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Endurance4Button",
                                                                NULL, rmOnSelectRaceManEndurance4);
    }
    else if (index == 8)
    {
        if(rmIndex == 0)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Now you can do a Championship season");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available :");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "Two races are available this weekend.the first on an Alicante track for 5 laps,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is a championship with race on 320 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Challenge Alicante");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Championship 320 Km");
        }
        else if (rmIndex == 1)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "Now you can do a Championship season");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available :");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first is a challenge at Alicante,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is a championship with race on 320 Km");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Challenge Alicante");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Championship");
        }

        Challenge1ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Challenge1Button",
                                                                NULL, rmOnSelectRaceManChallenge1);
        Championship3ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Championship3Button",
                                                                NULL, rmOnSelectRaceManChampionship3);
    }
    else if (index == 9)
    {
        if(rmIndex == 0)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "We are almost at the end of the season");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available :");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first is the Challenge at Brondehach,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is a 500 KM at Karwada");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Race at Brondehach");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Race at Karwada ");
        }
        else if (rmIndex == 1)
        {
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text1Label, "We are almost at the end of the season");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text2Label, "Races available :");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text3Label, "The first is the Challenge at Brondehach on 10 laps,");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Text4Label, "the second is a challenge at Forza on 10 laps");

            GfuiLabelSetText(RmCareerSelectMenuHandle, Race1Label, "Races at Brondehach");
            GfuiLabelSetText(RmCareerSelectMenuHandle, Race2Label, "Races at Forza");
        }

        Challenge5ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Challenge5Button",
                                                                NULL, rmOnSelectRaceManChallenge5);
        Challenge7ButtonId = GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "Endurance4Button",
                                                                NULL, rmOnSelectRaceManChallenge7);
    }

    // Create Back button
    GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "QuitButton",
                                prevMenu, rmOnRaceSelectShutdown);
    GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "BuyButton",
                                prevMenu, rmOnRaceSelectBuy);
    GarageButtonId =
            GfuiMenuCreateButtonControl(RmCareerSelectMenuHandle, hMenuXMLDesc, "GarageButton",
                                prevMenu, rmOnRaceSelectGarage);

    if(!rmCar)
        GfuiEnable(RmCareerSelectMenuHandle, GarageButtonId, GFUI_DISABLE);

    // Close menu XML descriptor.
    GfParmReleaseHandle(hMenuXMLDesc);
    
    // Register keyboard shortcuts.
    GfuiMenuDefaultKeysAdd(RmCareerSelectMenuHandle);
    GfuiAddKey(RmCareerSelectMenuHandle, GFUIK_ESCAPE, "Back To Main Menu",
               prevMenu, rmOnRaceSelectShutdown, NULL);

    // Give the race engine the menu to come back to.

    return RmCareerSelectMenuHandle;
}
